@@ -124,6 +124,7 @@ impl Assembler
 					fileserver,
 					filename,
 					None,
+					&mut std::collections::HashSet::new(),
 					&mut std::collections::HashSet::new());
 				
 				if pass_report.has_errors() || result.is_err()
@@ -1033,7 +1034,13 @@ impl State
 				"incbin" |
 				"incbinstr" |
 				"inchexstr" |
-				"le" =>
+				"le" |
+				"utf8" |
+				"utf16be" |
+				"utf16le" |
+				"utf32be" |
+				"utf32le" |
+				"ascii" =>
 				{
 					return Ok(expr::Value::Function(info.hierarchy[0].clone()));
 				}
@@ -1236,7 +1243,7 @@ impl State
 						State::eval_fn_check_arg_number(info, 1)?;
 						if State::eval_fn_check_unknown_arg(info, 0, self.is_first_pass)
 						{
-							return Ok(expr::Value::make_integer(util::BigInt::new_from_str("")));
+							return Ok(expr::Value::make_integer(util::BigInt::from_bytes_be(&[])));
 						}
 
 						let value_string = State::eval_fn_get_string_arg(info, 0)?;
@@ -1314,6 +1321,17 @@ impl State
 						}
 					}
 
+					"utf8" |
+					"utf16be" |
+					"utf16le" |
+					"utf32be" |
+					"utf32le" |
+					"ascii" =>
+					{
+						let s = State::eval_fn_get_string_arg(info, 0)?;
+						Ok(expr::Value::make_string(&s.utf8_contents, name))
+					}
+
 					_ => unreachable!()
 				}
 			}
@@ -1330,6 +1348,10 @@ impl State
 		fileserver: &dyn util::FileServer)
 		-> Result<expr::Value, ()>
 	{
+		// Clone the context in order to advance the logical address
+		// between instructions.
+		let mut inner_ctx = ctx.clone();
+
 		let mut result = util::BigInt::new(0, Some(0));
 		
 		let mut parser = syntax::Parser::new(Some(info.report.clone()), info.tokens);
@@ -1386,7 +1408,7 @@ impl State
 			let matches = asm::parser::match_rule_invocation(
 				&self,
 				subparser,
-				ctx.clone(),
+				inner_ctx.clone(),
 				fileserver,
 				info.report.clone())?;
 
@@ -1440,6 +1462,8 @@ impl State
 						&bigint,
 						(size, 0));
 				}
+				
+				inner_ctx.bit_offset += size;
 			}
 
 			parser.expect_linebreak()?;
